@@ -1,6 +1,6 @@
-use anyhow::Result;
-
 use super::{Audit, AuditLoadError, audit_meta};
+use crate::audit::AuditError;
+use crate::config::Config;
 use crate::finding::{Confidence, Finding, Severity};
 use crate::models::workflow::Workflow;
 use crate::state::AuditState;
@@ -13,12 +13,17 @@ audit_meta!(
     "use of fundamentally insecure workflow trigger"
 );
 
+#[async_trait::async_trait]
 impl Audit for DangerousTriggers {
-    fn new(_state: &AuditState<'_>) -> Result<Self, AuditLoadError> {
+    fn new(_state: &AuditState) -> Result<Self, AuditLoadError> {
         Ok(Self)
     }
 
-    fn audit_workflow<'doc>(&self, workflow: &'doc Workflow) -> Result<Vec<Finding<'doc>>> {
+    async fn audit_workflow<'doc>(
+        &self,
+        workflow: &'doc Workflow,
+        _config: &Config,
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
         if workflow.has_pull_request_target() {
             findings.push(
