@@ -8,10 +8,12 @@ use tree_sitter::{
 };
 
 use super::{Audit, AuditLoadError, audit_meta};
+use crate::audit::AuditError;
 use crate::config::Config;
 use crate::finding::location::Locatable as _;
 use crate::finding::{Confidence, Finding, Severity};
-use crate::models::{workflow::JobExt as _, workflow::Step};
+use crate::models::StepCommon;
+use crate::models::{workflow::JobCommon as _, workflow::Step};
 use crate::state::AuditState;
 use crate::utils;
 use crate::utils::once::static_regex;
@@ -86,20 +88,22 @@ const PWSH_REDIRECT_QUERY: &str = r#"
 
 const PWSH_PIPELINE_QUERY: &str = r#"
 (pipeline
-  (command
-    command_name: (command_name) @cmd
-    command_elements: (command_elements
-      (_)*
-      (array_literal_expression
-        (unary_expression [
-          (string_literal
-            (expandable_string_literal (variable) @destination))
-          (variable) @destination
-        ])
-      )
-      (_)*))
-  (#match? @cmd "(?i)out-file|add-content|set-content|tee-object")
-  (#match? @destination "(?i)ENV:GITHUB_ENV|ENV:GITHUB_PATH")
+  (pipeline_chain
+    (command
+        command_name: (command_name) @cmd
+        command_elements: (command_elements
+        (_)*
+        (array_literal_expression
+            (unary_expression [
+            (string_literal
+                (expandable_string_literal (variable) @destination))
+            (variable) @destination
+            ])
+        )
+        (_)*))
+    (#match? @cmd "(?i)out-file|add-content|set-content|tee-object")
+    (#match? @destination "(?i)ENV:GITHUB_ENV|ENV:GITHUB_PATH")
+  )
 ) @span
 "#;
 
@@ -146,17 +150,19 @@ impl GitHubEnv {
     fn bash_uses_github_env<'hay>(
         &self,
         script_body: &'hay str,
-    ) -> Result<Vec<(&'hay str, Range<usize>)>> {
+    ) -> Result<Vec<(&'hay str, Range<usize>)>, AuditError> {
         let mut parser = Parser::new();
         parser
             .set_language(&self.bash)
-            .context("failed to set bash language for parser")?;
+            .context("failed to set bash language for parser")
+            .map_err(Self::err)?;
 
         let mut cursor = QueryCursor::new();
 
         let tree = parser
             .parse(script_body, None)
-            .context("failed to parse `run:` body as bash")?;
+            .context("failed to parse `run:` body as bash")
+            .map_err(Self::err)?;
 
         // Look for redirect patterns, e.g. `... >> $GITHUB_ENV`.
         //
@@ -173,6 +179,10 @@ impl GitHubEnv {
             .bash_redirect_query
             .capture_index_for_name("args")
             .expect("internal error: missing capture index for 'args'");
+        let destination = self
+            .bash_redirect_query
+            .capture_index_for_name("destination")
+            .expect("internal error: missing capture index for 'destination'");
 
         let mut matching_spans = vec![];
 
@@ -203,7 +213,7 @@ impl GitHubEnv {
                     let cap = mat
                         .captures
                         .iter()
-                        .find(|cap| cap.index == self.bash_redirect_query.destination_idx)
+                        .find(|cap| cap.index == destination)
                         .expect("internal error: expected capture for destination");
                     cap.node
                         .utf8_text(script_body.as_bytes())
@@ -219,6 +229,9 @@ impl GitHubEnv {
         ];
 
         for query in queries {
+            let destination = query.capture_index_for_name("destination").expect(
+                "internal error: missing capture index for 'destination' in bash pipeline query",
+            );
             let matches = self.query(query, &mut cursor, &tree, script_body);
 
             matches.for_each(|mat| {
@@ -232,7 +245,7 @@ impl GitHubEnv {
                     let cap = mat
                         .captures
                         .iter()
-                        .find(|cap| cap.index == query.destination_idx)
+                        .find(|cap| cap.index == destination)
                         .expect("internal error: expected capture for destination");
                     cap.node
                         .utf8_text(script_body.as_bytes())
@@ -262,21 +275,26 @@ impl GitHubEnv {
     fn pwsh_uses_github_env<'hay>(
         &self,
         script_body: &'hay str,
-    ) -> Result<Vec<(&'hay str, Range<usize>)>> {
+    ) -> Result<Vec<(&'hay str, Range<usize>)>, AuditError> {
         let mut parser = Parser::new();
         parser
             .set_language(&self.pwsh)
-            .context("failed to set pwsh language for parser")?;
+            .context("failed to set pwsh language for parser")
+            .map_err(Self::err)?;
 
         let tree = parser
             .parse(script_body, None)
-            .context("failed to parse `run:` body as pwsh")?;
+            .context("failed to parse `run:` body as pwsh")
+            .map_err(Self::err)?;
 
         let mut cursor = QueryCursor::new();
         let queries = [&self.pwsh_redirect_query, &self.pwsh_pipeline_query];
         let mut matching_spans = vec![];
 
         for query in queries {
+            let destination = query
+                .capture_index_for_name("destination")
+                .expect("internal error: missing capture index for 'destination' in pwsh query");
             let matches = self.query(query, &mut cursor, &tree, script_body);
             matches.for_each(|mat| {
                 let span = mat
@@ -289,7 +307,7 @@ impl GitHubEnv {
                     let cap = mat
                         .captures
                         .iter()
-                        .find(|cap| cap.index == query.destination_idx)
+                        .find(|cap| cap.index == destination)
                         .expect("internal error: no matching capture");
                     cap.node
                         .utf8_text(script_body.as_bytes())
@@ -307,7 +325,7 @@ impl GitHubEnv {
         &self,
         run_step_body: &'hay str,
         shell: &str,
-    ) -> Result<Vec<(&'hay str, Range<usize>)>> {
+    ) -> Result<Vec<(&'hay str, Range<usize>)>, AuditError> {
         // The `shell:` stanza can contain a path and/or multiple arguments,
         // which we need to normalize out before comparing.
         // For example, `shell: /bin/bash -e {0}` becomes `bash`.
@@ -329,6 +347,7 @@ impl GitHubEnv {
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for GitHubEnv {
     fn new(_state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -358,11 +377,11 @@ impl Audit for GitHubEnv {
         })
     }
 
-    fn audit_step<'doc>(
+    async fn audit_step<'doc>(
         &self,
         step: &Step<'doc>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         let workflow = step.workflow();
@@ -375,10 +394,10 @@ impl Audit for GitHubEnv {
         }
 
         if let StepBody::Run { run, .. } = &step.deref().body {
-            let shell = step.shell().unwrap_or_else(|| {
+            let shell = step.shell().map(|s| s.0).unwrap_or_else(|| {
                 tracing::warn!(
-                    "github-env: couldn't determine shell type for {workflow}:{job} step {stepno}",
-                    workflow = step.workflow().key.filename(),
+                    "github-env: couldn't determine shell type for {workflow}:{job} step {stepno}; assuming bash",
+                    workflow = step.workflow().key.presentation_path(),
                     job = step.parent.id(),
                     stepno = step.index
                 );
@@ -402,7 +421,7 @@ impl Audit for GitHubEnv {
                                 .with_keys(["run".into()])
                                 .annotated(format!("write to {dest} may allow code execution")),
                         )
-                        .build(step.workflow())?,
+                        .build(step)?,
                 )
             }
         }
@@ -410,17 +429,30 @@ impl Audit for GitHubEnv {
         Ok(findings)
     }
 
-    fn audit_composite_step<'doc>(
+    async fn audit_composite_step<'doc>(
         &self,
         step: &super::CompositeStep<'doc>,
         _config: &Config,
-    ) -> Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
-        let action::StepBody::Run { run, shell, .. } = &step.body else {
+        let action::StepBody::Run { run, .. } = &step.body else {
             return Ok(findings);
         };
 
+        let shell = step.shell().map(|s| s.0).unwrap_or_else(|| {
+            tracing::warn!(
+                "github-env: couldn't determine shell type for {action} step {stepno}; assuming bash",
+                action = step.action().key.presentation_path(),
+                stepno = step.index
+            );
+
+            // The only way shell inference can fail for a `run:` in a
+            // composition action is if a user specifies an expression instead
+            // of a string literal. In that case, assume bash.
+            "bash"
+        });
+
         // TODO: actually use the spanning information here.
         for (dest, _span) in self.uses_github_env(run, shell)? {
             findings.push(
@@ -433,7 +465,7 @@ impl Audit for GitHubEnv {
                             .with_keys(["run".into()])
                             .annotated(format!("write to {dest} may allow code execution")),
                     )
-                    .build(step.action())?,
+                    .build(step)?,
             )
         }
 
