@@ -19,14 +19,12 @@ use std::{env, ops::Deref, sync::LazyLock, vec};
 
 use fst::Map;
 use github_actions_expressions::{Expr, context::Context, literal::Literal};
-use github_actions_models::{
-    common::{EnvValue, RepositoryUses, Uses, expr::LoE},
-    workflow::job::Strategy,
-};
+use github_actions_models::common::{EnvValue, RepositoryUses, Uses};
 use itertools::Itertools as _;
 
 use super::{Audit, AuditLoadError, audit_meta};
 use crate::{
+    audit::AuditError,
     config::Config,
     finding::{
         Confidence, Finding, Fix, Persona, Severity,
@@ -50,6 +48,7 @@ audit_meta!(
     "code injection via template expansion"
 );
 
+#[allow(clippy::unwrap_used)]
 static ACTION_INJECTION_SINKS: LazyLock<Vec<(RepositoryUsesPattern, Vec<&str>)>> =
     LazyLock::new(|| {
         let mut sinks: Vec<(RepositoryUsesPattern, Vec<&str>)> = serde_json::from_slice(
@@ -183,7 +182,7 @@ impl TemplateInjection {
             return None;
         }
 
-        let shell = utils::normalize_shell(step.shell()?);
+        let shell = utils::normalize_shell(step.shell()?.0);
 
         match shell {
             "bash" | "sh" | "zsh" => Some(format!("${{{env_var}}}")),
@@ -378,8 +377,8 @@ impl TemplateInjection {
                 Subfeature::new(expr_span.start, &parsed),
                 // Intentionally not providing a fix here.
                 None,
-                Severity::Unknown,
-                Confidence::Unknown,
+                Severity::Low,
+                Confidence::High,
                 Persona::Pedantic,
             ));
 
@@ -440,7 +439,7 @@ impl TemplateInjection {
                                             (Severity::High, Confidence::High, Persona::default())
                                         }
                                         None => {
-                                            (Severity::Unknown, Confidence::Low, Persona::default())
+                                            (Severity::Low, Confidence::Low, Persona::default())
                                         }
                                     };
 
@@ -477,7 +476,7 @@ impl TemplateInjection {
                                                 origin.raw,
                                             ),
                                             self.attempt_fix(&expr, &parsed, step),
-                                            Severity::Unknown,
+                                            Severity::Low,
                                             Confidence::High,
                                             Persona::Pedantic,
                                         ));
@@ -496,33 +495,25 @@ impl TemplateInjection {
                                         Persona::default(),
                                     ));
                                 } else if context.child_of("matrix") {
-                                    if let Some(Strategy { matrix, .. }) = step.strategy() {
-                                        let matrix_is_static = match matrix {
-                                            // The matrix is generated by an expression, meaning
-                                            // that it's trivially not static.
-                                            Some(LoE::Expr(_)) => false,
-                                            // The matrix may expand to static values according to the context
-                                            Some(inner) => models::workflow::Matrix::new(inner)
-                                                .expands_to_static_values(context),
-                                            // Context specifies a matrix, but there is no matrix defined.
-                                            // This is an invalid workflow so there's no point in flagging it.
-                                            None => continue,
-                                        };
+                                    let matrix_is_static = match step.matrix() {
+                                        Some(matrix) => matrix.expands_to_static_values(context),
+                                        // Context specifies a matrix, but there is no matrix defined.
+                                        // This is an invalid workflow so there's no point in flagging it.
+                                        None => continue,
+                                    };
 
-                                        if !matrix_is_static {
-                                            bad_expressions.push((
-                                                Subfeature::new(
-                                                    expr_span.start + origin.span.start,
-                                                    origin.raw,
-                                                ),
-                                                self.attempt_fix(&expr, &parsed, step),
-                                                Severity::Medium,
-                                                Confidence::Medium,
-                                                Persona::default(),
-                                            ));
-                                        }
+                                    if !matrix_is_static {
+                                        bad_expressions.push((
+                                            Subfeature::new(
+                                                expr_span.start + origin.span.start,
+                                                origin.raw,
+                                            ),
+                                            self.attempt_fix(&expr, &parsed, step),
+                                            Severity::Medium,
+                                            Confidence::Medium,
+                                            Persona::default(),
+                                        ));
                                     }
-                                    continue;
                                 } else {
                                     // All other contexts are typically not attacker controllable,
                                     // but may be in obscure cases.
@@ -562,7 +553,7 @@ impl TemplateInjection {
     fn process_step<'doc>(
         &self,
         step: &impl StepCommon<'doc>,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         let mut findings = vec![];
 
         for (script, script_loc, related_locs) in Self::scripts_with_location(step) {
@@ -599,6 +590,7 @@ impl TemplateInjection {
     }
 }
 
+#[async_trait::async_trait]
 impl Audit for TemplateInjection {
     fn new(_state: &AuditState) -> Result<Self, AuditLoadError>
     where
@@ -607,19 +599,19 @@ impl Audit for TemplateInjection {
         Ok(Self)
     }
 
-    fn audit_step<'doc>(
+    async fn audit_step<'doc>(
         &self,
         step: &Step<'doc>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'doc>>> {
+    ) -> Result<Vec<Finding<'doc>>, AuditError> {
         self.process_step(step)
     }
 
-    fn audit_composite_step<'a>(
+    async fn audit_composite_step<'a>(
         &self,
         step: &CompositeStep<'a>,
         _config: &Config,
-    ) -> anyhow::Result<Vec<Finding<'a>>> {
+    ) -> Result<Vec<Finding<'a>>, AuditError> {
         self.process_step(step)
     }
 }
@@ -639,11 +631,14 @@ mod tests {
     /// Macro for testing workflow audits with common boilerplate
     macro_rules! test_workflow_audit {
         ($audit_type:ty, $filename:expr, $workflow_content:expr, $test_fn:expr) => {{
-            let key = InputKey::local("fakegroup".into(), $filename, None::<&str>).unwrap();
+            let key = InputKey::local("fakegroup".into(), $filename, None::<&str>);
             let workflow = Workflow::from_string($workflow_content.to_string(), key).unwrap();
             let audit_state = AuditState::default();
             let audit = <$audit_type>::new(&audit_state).unwrap();
-            let findings = audit.audit_workflow(&workflow, &Config::default()).unwrap();
+            let findings = audit
+                .audit_workflow(&workflow, &Config::default())
+                .await
+                .unwrap();
 
             $test_fn(&workflow, findings)
         }};
@@ -666,8 +661,8 @@ mod tests {
         fix.apply(document).unwrap()
     }
 
-    #[test]
-    fn test_template_injection_fix_github_ref_name() {
+    #[tokio::test]
+    async fn test_template_injection_fix_github_ref_name() {
         let workflow_content = r#"
 name: Test Template Injection
 on: push
@@ -701,6 +696,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -715,8 +711,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_github_actor() {
+    #[tokio::test]
+    async fn test_template_injection_fix_github_actor() {
         let workflow_content = r#"
 name: Test Template Injection
 on: push
@@ -752,6 +748,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -768,8 +765,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_with_existing_env() {
+    #[tokio::test]
+    async fn test_template_injection_fix_with_existing_env() {
         let workflow_content = r#"
 name: Test Template Injection
 on: push
@@ -805,6 +802,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection
                     on: push
                     jobs:
@@ -822,8 +820,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_no_fix_for_action_sinks() {
+    #[tokio::test]
+    async fn test_template_injection_no_fix_for_action_sinks() {
         let workflow_content = r#"
 name: Test Template Injection - Actions
 on: push
@@ -857,8 +855,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_multiple_expressions() {
+    #[tokio::test]
+    async fn test_template_injection_fix_multiple_expressions() {
         let workflow_content = r#"
 name: Test Multiple Template Injections
 on: push
@@ -914,6 +912,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Multiple Template Injections
                 on: push
                 jobs:
@@ -933,8 +932,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_duplicate_expressions() {
+    #[tokio::test]
+    async fn test_template_injection_fix_duplicate_expressions() {
         let workflow_content = r#"
         name: Test Duplicate Template Injections
         on: push
@@ -980,6 +979,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -996,8 +996,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_equivalent_expressions() {
+    #[tokio::test]
+    async fn test_template_injection_fix_equivalent_expressions() {
         let workflow_content = r#"
         name: Test Duplicate Template Injections
         on: push
@@ -1040,6 +1040,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -1057,8 +1058,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_no_env_overcorrection() {
+    #[tokio::test]
+    async fn test_template_injection_fix_no_env_overcorrection() {
         // Testcase for #1052.
         let workflow_content = r#"
 name: Test Duplicate Template Injections
@@ -1103,6 +1104,7 @@ jobs:
                 }
 
                 insta::assert_snapshot!(current_document.source(), @r#"
+
                 name: Test Duplicate Template Injections
                 on: push
                 jobs:
@@ -1188,8 +1190,8 @@ jobs:
         }
     }
 
-    #[test]
-    fn test_template_injection_fix_bash_shell() {
+    #[tokio::test]
+    async fn test_template_injection_fix_bash_shell() {
         let workflow_content = r#"
 name: Test Template Injection - Bash
 on: push
@@ -1219,6 +1221,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Bash
                     on: push
                     jobs:
@@ -1234,8 +1237,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_bash_shell_full_path() {
+    #[tokio::test]
+    async fn test_template_injection_fix_bash_shell_full_path() {
         let workflow_content = r#"
 name: Test Template Injection - Bash
 on: push
@@ -1265,6 +1268,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Bash
                     on: push
                     jobs:
@@ -1280,8 +1284,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_cmd_shell() {
+    #[tokio::test]
+    async fn test_template_injection_fix_cmd_shell() {
         let workflow_content = r#"
 name: Test Template Injection - CMD
 on: push
@@ -1310,7 +1314,8 @@ jobs:
                         finding,
                         "replace expression with environment variable",
                     );
-                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    insta::assert_snapshot!(fixed_content.source(), @r"
+
                     name: Test Template Injection - CMD
                     on: push
                     jobs:
@@ -1320,14 +1325,14 @@ jobs:
                           - name: Vulnerable step with cmd shell
                             shell: cmd
                             run: echo User is %GITHUB_ACTOR%
-                    "#);
+                    ");
                 }
             }
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_pwsh_shell() {
+    #[tokio::test]
+    async fn test_template_injection_fix_pwsh_shell() {
         let workflow_content = r#"
 name: Test Template Injection - PowerShell
 on: push
@@ -1357,6 +1362,7 @@ jobs:
                         "replace expression with environment variable",
                     );
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - PowerShell
                     on: push
                     jobs:
@@ -1372,8 +1378,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_default_shell_ubuntu() {
+    #[tokio::test]
+    async fn test_template_injection_fix_default_shell_ubuntu() {
         let workflow_content = r#"
 name: Test Template Injection - Default Shell Ubuntu
 on: push
@@ -1403,6 +1409,7 @@ jobs:
                     );
                     // Ubuntu default shell is bash, so should use ${VAR} syntax
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Default Shell Ubuntu
                     on: push
                     jobs:
@@ -1417,8 +1424,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_default_shell_windows() {
+    #[tokio::test]
+    async fn test_template_injection_fix_default_shell_windows() {
         let workflow_content = r#"
 name: Test Template Injection - Default Shell Windows
 on: push
@@ -1448,6 +1455,7 @@ jobs:
                     );
                     // Windows default shell is pwsh, so should use $env:VAR syntax
                     insta::assert_snapshot!(fixed_content.source(), @r#"
+
                     name: Test Template Injection - Default Shell Windows
                     on: push
                     jobs:
@@ -1462,8 +1470,8 @@ jobs:
         );
     }
 
-    #[test]
-    fn test_template_injection_fix_cmd_shell_with_custom_env() {
+    #[tokio::test]
+    async fn test_template_injection_fix_cmd_shell_with_custom_env() {
         let workflow_content = r#"
 name: Test Template Injection - CMD with Custom Env
 on: push
@@ -1492,7 +1500,8 @@ jobs:
                         finding,
                         "replace expression with environment variable",
                     );
-                    insta::assert_snapshot!(fixed_content.source(), @r#"
+                    insta::assert_snapshot!(fixed_content.source(), @r"
+
                     name: Test Template Injection - CMD with Custom Env
                     on: push
                     jobs:
@@ -1504,14 +1513,14 @@ jobs:
                             run: echo PR title is %GITHUB_EVENT_PULL_REQUEST_TITLE%
                             env:
                               GITHUB_EVENT_PULL_REQUEST_TITLE: ${{ github.event.pull_request.title }}
-                    "#);
+                    ");
                 }
             }
         );
     }
 
-    #[test]
-    fn test_template_injection_no_fix_unknown_shell() {
+    #[tokio::test]
+    async fn test_template_injection_no_fix_unknown_shell() {
         let workflow_content = r#"
 name: Test Template Injection - Unknown Shell
 on: push
