@@ -7,7 +7,7 @@ fn test_regular_persona() -> anyhow::Result<()> {
         zizmor()
             .input(input_under_test("anonymous-definition.yml"))
             .run()?,
-        @r"No findings to report. Good job! (2 suppressed)"
+        @"No findings to report. Good job! (2 suppressed)"
     );
 
     Ok(())
@@ -34,17 +34,16 @@ fn test_pedantic_persona() -> anyhow::Result<()> {
        | |__________________________________________^ this workflow
        |
        = note: audit confidence → High
+       = tip: use 'name: ...' to give this workflow a name
 
     info[anonymous-definition]: workflow or action definition without a name
       --> @@INPUT@@:21:3
        |
-    21 | /   will-trigger:
-    22 | |     runs-on: ubuntu-latest
-    23 | |     steps:
-    24 | |       - run: "echo this job will trigger"
-       | |__________________________________________^ this job
+    21 |   will-trigger:
+       |   ^^^^^^^^^^^^ this job
        |
        = note: audit confidence → High
+       = tip: use 'name: ...' to give this job a name
 
     2 findings: 1 informational, 1 low, 0 medium, 0 high
     "#
