@@ -83,13 +83,21 @@ fn menagerie() -> Result<()> {
 
 #[test]
 fn color_control_basic() -> Result<()> {
-    // No terminal, so no color by default.
+    // No terminal and not CI, so no color by default.
     let no_color_default_output = zizmor()
         .output(OutputMode::Both)
         .input(input_under_test("e2e-menagerie"))
         .run()?;
     assert!(!no_color_default_output.contains("\x1b["));
 
+    // No terminal but CI, so color by default.
+    let color_default_ci_output = zizmor()
+        .setenv("CI", "true")
+        .output(OutputMode::Both)
+        .input(input_under_test("e2e-menagerie"))
+        .run()?;
+    assert!(color_default_ci_output.contains("\x1b["));
+
     // Force color via --color=always.
     let forced_color_via_arg_output = zizmor()
         .output(OutputMode::Both)
@@ -251,6 +259,75 @@ fn invalid_inputs() -> Result<()> {
         );
     }
 
+    insta::assert_snapshot!(
+        zizmor()
+            .expects_failure(true)
+            .input(input_under_test("invalid/empty/"))
+            .args(["--strict-collection"])
+            .run()?,
+        @r"
+    🌈 zizmor v@@VERSION@@
+    fatal: no audit was performed
+    error: no inputs collected
+      |
+      = help: collection yielded no auditable inputs
+      = help: inputs must contain at least one valid workflow, action, or Dependabot config
+
+    Caused by:
+        no inputs collected
+    "
+    );
+
+    Ok(())
+}
+
+/// Reproduction test for #1395.
+///
+/// Ensures that we produce a useful error message when the user gives us an
+/// invalid YAML input (specifically, one with duplicate mapping keys).
+#[test]
+fn test_issue_1394() -> Result<()> {
+    insta::assert_snapshot!(
+        zizmor()
+            .expects_failure(true)
+            .input(input_under_test(
+                "invalid/issue-1395-repro-duplicate-mapping-keys.yml"
+            ))
+            .args(["--strict-collection"])
+            .run()?,
+        @r#"
+    🌈 zizmor v@@VERSION@@
+    fatal: no audit was performed
+    failed to load file://@@INPUT@@ as workflow
+
+    Caused by:
+        0: invalid YAML syntax: jobs.demo.steps[0]: duplicate entry with key "env" at line 10 column 9
+        1: jobs.demo.steps[0]: duplicate entry with key "env" at line 10 column 9
+    "#
+    );
+
+    // Without --strict-collection, we get a warning and then a collection failure error.
+    insta::assert_snapshot!(
+        zizmor()
+            .expects_failure(true)
+            .input(input_under_test(
+                "invalid/issue-1395-repro-duplicate-mapping-keys.yml"
+            ))
+            .run()?,
+        @r#"
+    🌈 zizmor v@@VERSION@@
+     WARN collect_inputs: zizmor::registry::input: failed to parse input: jobs.demo.steps[0]: duplicate entry with key "env" at line 10 column 9
+    fatal: no audit was performed
+    error: no inputs collected
+      |
+      = help: collection yielded no auditable inputs
+      = help: inputs must contain at least one valid workflow, action, or Dependabot config
+
+    Caused by:
+        no inputs collected
+    "#
+    );
+
     Ok(())
 }
 
@@ -342,10 +419,10 @@ fn issue_1065() -> Result<()> {
        = note: audit confidence → Medium
 
     error[unpinned-uses]: unpinned action reference
-      --> @@INPUT@@:16:9
+      --> @@INPUT@@:16:15
        |
     16 |         uses: thollander/actions-comment-pull-request@v3
-       |         ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ action is not pinned to a hash (required by blanket policy)
+       |               ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ action is not pinned to a hash (required by blanket policy)
        |
        = note: audit confidence → High
 
@@ -437,10 +514,10 @@ fn issue_1286() -> Result<()> {
         @r"
     🌈 zizmor v@@VERSION@@
     fatal: no audit was performed
-    ref-confusion failed on file://@@INPUT@@
+    'ref-confusion' audit failed on file://@@INPUT@@
 
     Caused by:
-        0: error in ref-confusion
+        0: error in 'ref-confusion' audit
         1: couldn't list branches for woodruffw-experiments/this-does-not-exist
         2: can't access woodruffw-experiments/this-does-not-exist: missing or you have no access
     ",
@@ -531,7 +608,6 @@ fn test_cant_retrieve_offline() -> Result<()> {
         zizmor()
             .expects_failure(true)
             .offline(true)
-            .unsetenv("GH_TOKEN")
             .args(["pypa/sampleproject"])
             .run()?,
         @r"
@@ -557,7 +633,7 @@ fn test_cant_retrieve_no_gh_token() -> Result<()> {
         zizmor()
             .expects_failure(true)
             .offline(false)
-            .unsetenv("GH_TOKEN")
+            .gh_token(false)
             .args(["pypa/sampleproject"])
             .run()?,
         @r"
@@ -595,3 +671,25 @@ fn test_github_output() -> Result<()> {
 
     Ok(())
 }
+
+/// Ensures that the `--show-audit-urls` flag works as expected.
+#[test]
+fn test_show_urls() -> Result<()> {
+    let with_urls = zizmor()
+        .offline(true)
+        .show_audit_urls(true)
+        .input(input_under_test("several-vulnerabilities.yml"))
+        .run()?;
+
+    assert!(with_urls.contains("audit documentation → "));
+
+    let without_urls = zizmor()
+        .offline(true)
+        .show_audit_urls(false)
+        .input(input_under_test("several-vulnerabilities.yml"))
+        .run()?;
+
+    assert!(!without_urls.contains("audit documentation → "));
+
+    Ok(())
+}
